import pygame, random, math, mmFragment

#states    
#stand    0
#move     1
#swing    2
#shaking  3

img_enemys = []
enemys = []

class Enemy:
	def __init__(self, images, pos, id):
		self.id = id
		self.img = images
		self.rect = self.img[0][0].get_rect()
		self.rect.x, self.rect.y = pos
		self.dir = 1
		self.state = 0
		self.shake_vec = [0,0]
		self.last_time = pygame.time.get_ticks()
		self.swing_time = 0
		self.delay_time = 0
		self.wait_time = 0
		self.walk_time = 0
		self.speed = 0.1
		self.frame = 0
		self.sframe = 0
		self.anim_rate = [0, 0.8, 0, 0]
		self.health = 2
		self.isAttacking = False
		
	def align(self, point, ends):
		if self.rect.x < point[0]:
			rx = point[0]
			ry = ends[0][1] - point[1]
			ex = self.rect.x
			self.rect.y = ends[0][1] - ((ry*ex)/rx) + 20
				
		elif self.rect.x > point[0]:
			rx = 800 - point[0]
			ry = ends[0][1] - point[1]
			ex = 800 - self.rect.x
			self.rect.y = ends[0][1] - ((ry*ex)/rx) + 20
			
	def update(self, point,ends, player_pos, shaking):
		
		delta = pygame.time.get_ticks() - self.last_time
		
		if not shaking:
			if self.state is 3:
				self.delay_time = 1500
				self.state = 0
				self.shake_vec[0] = 0
				self.shake_vec[1] = 0
		else:
			if self.state is not 3:
				self.health -= 1
				self.state = 3
				self.delay_time = 300
				
		if self.state is 0:
			if self.delay_time == 0:
				if abs(player_pos[0]-self.rect.x) < 120:
					self.start_swing()
				if abs(player_pos[0]-self.rect.x) > 150:
					self.state = 1
		elif self.state is 1:
			if abs(player_pos[0]-self.rect.x) < 111:
				self.state = 0
			if self.dir == 1:
				self.rect.x -= self.speed*delta
			if self.dir == 0:
				self.rect.x += self.speed*delta
				
		elif self.state is 2:
			if pygame.time.get_ticks() - self.swing_time > 500:
				self.state = 0
				self.delay_time = 1000
		elif self.state is 3:
			self.shake_vec[0] = random.randrange(10)
			self.shake_vec[1] = (random.randrange(100)-50) + 20
			if self.delay_time == 0:
				#DIE
				if self.health <= 0:
					mmFragment.fraglist.append(mmFragment.Fragment(pygame.transform.flip(self.img[3][self.frame],self.dir, 0), (self.rect.x-(self.rect.w/2) + self.shake_vec[0],self.rect.y-self.rect.h + self.shake_vec[1]), (0,-0.5), 0,0, (0,0.005)))
					return 1
		
		
		self.align(point, ends)
		
		if self.state is not 3:
			if self.rect.x < player_pos[0]:
				self.dir = 0
			elif self.rect.x > player_pos[0]:
				self.dir = 1
				
		#animation
		self.sframe += self.anim_rate[self.state] * delta
		if self.sframe > 100:
			self.sframe = 0
			self.frame += 1
		if self.frame > len(self.img[self.state])-1:
			self.frame = 0
				
		self.delay_time -= delta
		if self.delay_time < 0:
			self.delay_time = 0
			
			
		self.last_time = pygame.time.get_ticks()
		
		return 0
		
	def start_swing(self):
		self.state = 2
		self.swing_time = pygame.time.get_ticks()
		
			
				
	def predraw(self):
		return (self.rect.x-(self.rect.w/2) + self.shake_vec[0], self.rect.y-self.rect.h  + self.shake_vec[1], self.rect.w, self.rect.h)
	def draw(self):
		pygame.display.get_surface().blit(pygame.transform.flip(self.img[self.state][self.frame], self.dir, 0), (self.rect.x-(self.rect.w/2) + self.shake_vec[0], self.rect.y-self.rect.h  + self.shake_vec[1]))
	def getAttack(self):
		#      origin   lenth  dir
		return self.rect.x, 120, self.dir, self.rect.y
	

class RopeWalker:
	def __init__(self, pos, id):
		self.id = id
		self.img = img_enemys[0]
		self.rect = self.img[0][0].get_rect()
		self.rect.x, self.rect.y = pos
		self.dir = 1
		self.state = 0
		self.shake_vec = [0,0]
		self.last_time = pygame.time.get_ticks()
		self.swing_time = 0
		self.delay_time = 0
		self.wait_time = 0
		self.walk_time = 0
		self.speed = 0.1
		self.frame = 0
		self.sframe = 0
		self.anim_rate = [0, 0.8, 0, 0]
		self.health = 2
		self.isAttacking = False
		
	def align(self, point, ends):
		if self.rect.x < point[0]:
			rx = point[0]
			ry = ends[0][1] - point[1]
			ex = self.rect.x
			self.rect.y = ends[0][1] - ((ry*ex)/rx)  + 8
				
		elif self.rect.x > point[0]:
			rx = 800 - point[0]
			ry = ends[0][1] - point[1]
			ex = 800 - self.rect.x
			self.rect.y = ends[0][1] - ((ry*ex)/rx) + 8
			
	def update(self, point,ends, player_pos, shaking):
		
		delta = pygame.time.get_ticks() - self.last_time
		
		if not shaking:
			if self.state is 3:
				self.delay_time = 1500
				self.state = 0
				self.shake_vec[0] = 0
				self.shake_vec[1] = 0
		else:
			if self.state is not 3:
				self.health -= 1
				self.state = 3
				self.delay_time = 300
				
		if self.state is 0:
			if self.delay_time == 0:
				if abs(player_pos[0]-self.rect.x) < 120:
					self.start_swing()
				if abs(player_pos[0]-self.rect.x) > 150:
					self.state = 1
		elif self.state is 1:
			if abs(player_pos[0]-self.rect.x) < 111:
				self.state = 0
			if self.dir == 1:
				self.rect.x -= self.speed*delta
			if self.dir == 0:
				self.rect.x += self.speed*delta
				
		elif self.state is 2:
			self.isAttacking = True
			if pygame.time.get_ticks() - self.swing_time > 500:
				self.state = 0
				self.delay_time = 1000
				self.isAttacking = False
		elif self.state is 3:
			self.shake_vec[0] = random.randrange(10)
			self.shake_vec[1] = (random.randrange(100)-50) + 20
			if self.delay_time == 0:
				#DIE
				if self.health <= 0:
					mmFragment.fraglist.append(mmFragment.Fragment(pygame.transform.flip(self.img[3][self.frame],self.dir, 0), (self.rect.x-(self.rect.w/2) + self.shake_vec[0],self.rect.y-self.rect.h + self.shake_vec[1]), (0,-0.5), 0,0, (0,0.005)))
					return 1
		
		
		self.align(point, ends)
		
		if self.state not in [2,3]:
			if self.rect.x < player_pos[0]:
				self.dir = 0
			elif self.rect.x > player_pos[0]:
				self.dir = 1
				
		#animation
		self.sframe += self.anim_rate[self.state] * delta
		if self.sframe > 100:
			self.sframe = 0
			self.frame += 1
		if self.frame > len(self.img[self.state])-1:
			self.frame = 0
				
		self.delay_time -= delta
		if self.delay_time < 0:
			self.delay_time = 0
			
			
		self.last_time = pygame.time.get_ticks()
		
		return 0
		
	def start_swing(self):
		self.state = 2
		self.swing_time = pygame.time.get_ticks()
		
			
				
	def predraw(self):
		return (self.rect.x-(self.rect.w/2) + self.shake_vec[0], self.rect.y-self.rect.h  + self.shake_vec[1], self.rect.w, self.rect.h)
	def draw(self):
		pygame.display.get_surface().blit(pygame.transform.flip(self.img[self.state][self.frame], self.dir, 0), (self.rect.x-(self.rect.w/2) + self.shake_vec[0], self.rect.y-self.rect.h  + self.shake_vec[1]))
	def getAttack(self):
		#      origin   lenth  dir
		return self.rect.x, 120, self.dir, self.rect.y
	



class Hanger:
	def __init__(self, pos, id):
		self.id = id
		self.img = img_enemys[1]
		self.rect = self.img[0][0].get_rect()
		self.rect.x, self.rect.y = pos
		self.dir = 1
		self.state = 0
		self.shake_vec = [0,0]
		self.last_time = pygame.time.get_ticks()
		self.swing_time = 0
		self.delay_time = 0
		self.wait_time = 0
		self.walk_time = 0
		self.speed = 0.1
		self.frame = 0
		self.sframe = 0
		self.anim_rate = [0, 0.8, 1.0, 0, 0]
		self.health = 2
		self.isAttacking = False
		
	def align(self, point, ends):
		if self.rect.x < point[0]:
			rx = point[0]
			ry = ends[0][1] - point[1]
			ex = self.rect.x
			self.rect.y = ends[0][1] - ((ry*ex)/rx) + 100
				
		elif self.rect.x > point[0]:
			rx = 800 - point[0]
			ry = ends[0][1] - point[1]
			ex = 800 - self.rect.x
			self.rect.y = ends[0][1] - ((ry*ex)/rx) + 100
			
	def update(self, point,ends, player_pos, shaking):
		
		delta = pygame.time.get_ticks() - self.last_time
		
		if not shaking:
			if self.state is 3:
				self.delay_time = 1000
				self.state = 4
				self.shake_vec[0] = 0
				self.shake_vec[1] = 0
		else:
			if self.state is not 3:
				self.health -= 1
				self.state = 3
				self.frame = 0
				
		if self.state is 0:
			if self.delay_time == 0:
				if abs(player_pos[0]-self.rect.x) < 120:
					self.start_swing()
				if abs(player_pos[0]-self.rect.x) > 150:
					self.state = 1
		elif self.state is 1:
			if abs(player_pos[0]-self.rect.x) < 111:
				self.state = 0
			if self.dir == 1:
				self.rect.x -= self.speed*delta
			if self.dir == 0:
				self.rect.x += self.speed*delta
				
		elif self.state is 2:
			if pygame.time.get_ticks() - self.swing_time > 100:
				self.isAttacking = True
			if pygame.time.get_ticks() - self.swing_time > 400:
				self.state = 0
				self.isAttacking = False
		elif self.state is 3:
			self.shake_vec[0] = random.randrange(10)
			self.shake_vec[1] = (random.randrange(100)-50) + 20
			
					
		elif self.state is 4:
			#shaken, after vibration
			if self.delay_time == 0:
				self.state = 0
			if abs(player_pos[0]-self.rect.x) < 60:
				mmFragment.fraglist.append(mmFragment.Fragment(pygame.transform.flip(self.img[3][self.frame],self.dir, 0), (self.rect.x-(self.rect.w/2) + self.shake_vec[0],self.rect.y-self.rect.h + self.shake_vec[1]), (0,0), 0,0, (0,0.001)))
				return 1
				
				
			
		
		
		self.align(point, ends)
		
		if self.state not in [3,4]:
			if self.rect.x < player_pos[0]:
				self.dir = 0
			elif self.rect.x > player_pos[0]:
				self.dir = 1
				
		#animation
		self.sframe += self.anim_rate[self.state] * delta
		if self.sframe > 100:
			self.sframe = 0
			self.frame += 1
		if self.frame > len(self.img[self.state])-1:
			self.frame = 0
				
		self.delay_time -= delta
		if self.delay_time < 0:
			self.delay_time = 0
			
			
		self.last_time = pygame.time.get_ticks()
		
		return 0
		
	def start_swing(self):
		self.state = 2
		self.swing_time = pygame.time.get_ticks()
		
			
				
	def predraw(self):
		return (self.rect.x-(self.rect.w/2) + self.shake_vec[0], self.rect.y-self.rect.h  + self.shake_vec[1], self.rect.w, self.rect.h)
	def draw(self):
		pygame.display.get_surface().blit(pygame.transform.flip(self.img[self.state][self.frame], self.dir, 0), (self.rect.x-(self.rect.w/2) + self.shake_vec[0], self.rect.y-self.rect.h  + self.shake_vec[1]))
	def getAttack(self):
		#      origin   lenth  dir
		return self.rect.x, 70, self.dir, self.rect.y
		
		